#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update Distributed Cache Service Instance
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


params = [
    {
        'param_id': '1',
        'param_name': 'timeout',
        'default_value': '0',
        'value_range': '0-7200',
        'value_type': 'Interger',
        'param_value': '11'
    },
    {
        'param_id': '2',
        'param_name': 'maxmemory-policy',
        'default_value': 'noeviction',
        'value_range':
            'volatile-lru,allkeys-lru,volatile-random,'
            'allkeys-random,volatile-ttl,noeviction',
        'value_type': 'Enum',
        'param_value': 'allkeys-random'
    }
]
instance = 'name_or_id'
instance = conn.dcs.find_instance(instance)
conn.dcs.update_instance_params(
    instance=instance,
    params=params
)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.tms.v1 import _proxy
from otcextensions.sdk.tms.v1 import resource_tag
from otcextensions.sdk.tms.v1 import predefined_tag
from unittest.mock import MagicMock


class TestTmsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestTmsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestTmsPredefinedTag(TestTmsProxy):
    def test_predefined_tags(self):
        self.verify_list(self.proxy.predefined_tags,
                         predefined_tag.PredefinedTag)

    def test_predefined_tag_create(self):
        self._verify(
            mock_method='otcextensions.sdk.tms.v1.predefined_tag.'
                        'PredefinedTag.add_tag',
            test_method=self.proxy.create_predefined_tag,
            method_kwargs={
                'key': 'key',
                'value': 'value'
            },
            expected_kwargs={
                'key': 'key',
                'value': 'value'
            },
            expected_args=[
                self.proxy,
            ]
        )

    def test_predefined_tag_delete(self):
        self._verify(
            mock_method='otcextensions.sdk.tms.v1.predefined_tag.'
                        'PredefinedTag.delete_tag',
            test_method=self.proxy.delete_predefined_tag,
            method_kwargs={
                'key': 'key',
                'value': 'value'
            },
            expected_kwargs={
                'key': 'key',
                'value': 'value'
            },
            expected_args=[
                self.proxy,
            ]
        )

    def test_predefined_tag_update(self):
        self.verify_update(self.proxy.update_predefined_tag,
                           predefined_tag.PredefinedTag,
                           method_args=[],
                           method_kwargs={
                               "new_tag": {
                                   "key": "key",
                                   "value": "value"},
                               "old_tag": {
                                   'key': 'key',
                                   'value': 'new_value'
                               }
                           },
                           expected_kwargs={
                               "new_tag": {
                                   "key": "key",
                                   "value": "value"},
                               "old_tag": {
                                   'key': 'key',
                                   'value': 'new_value'
                               }
                           }, )


class TestTmsResourceTag(TestTmsProxy):
    def test_resource_tag(self):
        self.proxy._get_endpoint_with_api_version = MagicMock(
            return_value="http://test")
        self.verify_list(self.proxy.resource_tags,
                         resource_tag.ResourceTag,
                         method_args=['resource_id', 'resource_type'],
                         expected_args=[])

    def test_resource_tag_create(self):
        self.verify_create(self.proxy.create_resource_tag,
                           resource_tag.ResourceTag,
                           expected_args=[],
                           method_kwargs={
                               "project_id": "id",
                               "resources": [{"resource_id": "id",
                                              "resource_type": "vpc"}],
                               "tags": [{"key": "ENV1", "value": "dev1"}],
                           },
                           expected_kwargs={
                               "project_id": "id",
                               "resources": [{"resource_id": "id",
                                              "resource_type": "vpc"}],
                               "tags": [{"key": "ENV1", "value": "dev1"}],
                           },
                           )

    def test_resource_tag_delete(self):
        self.verify_delete(self.proxy.delete_resource_tag,
                           resource_tag.ResourceTag,
                           method_args=[],
                           method_kwargs={
                               "project_id": "id",
                               "resources": [{"resource_id": "id",
                                              "resource_type": "vpc"}],
                               "tags": [{"key": "ENV1", "value": "dev1"}],
                           },
                           expected_kwargs={
                               "project_id": "id",
                               "resources": [{"resource_id": "id",
                                              "resource_type": "vpc"}],
                               "tags": [{"key": "ENV1", "value": "dev1"}],
                           },)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Vpc(resource.Resource):
    resource_key = 'vpc'
    resources_key = 'vpcs'
    base_path = '/v1/%(project_id)s/vpcs'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    project_id = resource.URI('project_id')
    #: Specifies the available IP address ranges for subnets in the VPC.
    cidr = resource.Body('cidr')
    #: Provides supplementary information about the VPC.
    description = resource.Body('description')
    #: Specifies whether the shared SNAT function is enabled.
    enable_shared_snat = resource.Body('enable_shared_snat', type=bool)
    #: Specifies the VPC name.
    name = resource.Body('name')
    #: Specifies the route list.
    routes = resource.Body('routes', type=list, list_type=dict)
    #: Specifies the VPC status.
    status = resource.Body('status')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.identity.v3 import _bad_base as _base


class SecurityToken(_base.BadBaseResource):
    resource_key = 'credential'
    base_path = '/v3.0/OS-CREDENTIAL/securitytokens'

    # capabilities
    allow_create = True

    # Properties
    #: Authorization data (used to request token)
    auth = resource.Body('auth', type=dict)
    #: Access Key
    access = resource.Body('access')
    #: Expiration time
    expires_at = resource.Body('expires_at')
    #: Secret Key
    secret = resource.Body('secret')
    #: Security Token
    security_token = resource.Body('securitytoken')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.common import cli_utils
from otcextensions.osclient.css.v1 import cluster
from otcextensions.tests.unit.osclient.css.v1 import fakes

COLUMNS = (
    'action_progress',
    'actions',
    'bandwidth_size',
    'cmk_id',
    'created_at',
    'datastore',
    'elb_whitelist',
    'endpoints',
    'enterprise_project_id',
    'floating_ip',
    'id',
    'is_authority_enabled',
    'is_backup_enabled',
    'is_billed',
    'is_disk_encrypted',
    'is_https_enabled',
    'name',
    'network_id',
    'nodes',
    'router_id',
    'security_group_id',
    'status',
    'status_code',
    'tags',
    'updated_at',
)


class TestListClusters(fakes.TestCss):
    _data = fakes.FakeCluster.create_multiple(3)

    column_list_headers = (
        'ID',
        'Name',
        'Type',
        'Version',
        'Status',
        'Created At',
    )

    columns = ('id', 'name', 'type', 'version', 'status', 'created_at')

    data = []

    for s in _data:
        data.append(
            (
                s.id,
                s.name,
                s.datastore['type'],
                s.datastore['version'],
                s.status,
                s.created_at,
            )
        )

    def setUp(self):
        super(TestListClusters, self).setUp()

        self.cmd = cluster.ListClusters(self.app, None)

        self.client.clusters = mock.Mock()

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.clusters.side_effect = [self._data]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.clusters.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateCluster(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    columns = COLUMNS

    data = fakes.gen_data(_data, columns)

    default_timeout = 1200

    def setUp(self):
        super(TestCreateCluster, self).setUp()

        self.cmd = cluster.CreateCluster(self.app, None)

        self.client.create_cluster = mock.Mock(return_value=self._data)
        self.client.get_cluster = mock.Mock(return_value=self._data)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_create(self):
        arglist = [
            'test-css',
            '--flavor',
            'css-flavor',
            '--datastore-version',
            '7.10.2',
            '--availability-zone',
            'eu-de-01,eu-de-02',
            '--router-id',
            'router-uuid',
            '--network-id',
            'network-uuid',
            '--security-group-id',
            'sg-uuid',
            '--num-nodes',
            '2',
            '--volume-size',
            '100',
            '--volume-type',
            'COMMON',
            '--cmk-id',
            'cmk-uuid',
            '--https-enable',
            '--admin-pwd',
            'testtest',
            '--backup-policy',
            (
                'period=00:00 GMT+08:00,keepday=7,prefix=snapshot,'
                'bucket=css-backup-bucket,agency=obs_css_agency,'
                'basepath=obs_path'
            ),
            '--tag',
            'key=key1,value=value1',
            '--tag',
            'key=key2,value=value2',
            '--wait',
        ]
        verifylist = [
            ('name', 'test-css'),
            ('flavor', 'css-flavor'),
            ('datastore_version', '7.10.2'),
            ('availability_zone', 'eu-de-01,eu-de-02'),
            ('router_id', 'router-uuid'),
            ('network_id', 'network-uuid'),
            ('security_group_id', 'sg-uuid'),
            ('num_nodes', 2),
            ('volume_size', 100),
            ('volume_type', 'COMMON'),
            ('cmk_id', 'cmk-uuid'),
            ('https_enable', True),
            ('admin_pwd', 'testtest'),
            (
                'backup_policy',
                [
                    {
                        'period': '00:00 GMT+08:00',
                        'keepday': '7',
                        'prefix': 'snapshot',
                        'bucket': 'css-backup-bucket',
                        'agency': 'obs_css_agency',
                        'basepath': 'obs_path',
                    }
                ],
            ),
            (
                'tags',
                [
                    {'key': 'key1', 'value': 'value1'},
                    {'key': 'key2', 'value': 'value2'},
                ],
            ),
            ('wait', True),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            'name': 'test-css',
            'datastore': {'version': '7.10.2', 'type': 'elasticsearch'},
            'instanceNum': 2,
            'instance': {
                'availability_zone': 'eu-de-01,eu-de-02',
                'flavorRef': 'css-flavor',
                'volume': {'volume_type': 'COMMON', 'size': 100},
                'nics': {
                    'vpcId': 'router-uuid',
                    'netId': 'network-uuid',
                    'securityGroupId': 'sg-uuid',
                },
            },
            'diskEncryption': {
                'systemEncrypted': 1,
                'systemCmkid': 'cmk-uuid',
            },
            'httpsEnable': True,
            'authorityEnable': True,
            'adminPwd': 'testtest',
            'backupStrategy': {
                'period': '00:00 GMT+08:00',
                'keepday': 7,
                'prefix': 'snapshot',
                'bucket': 'css-backup-bucket',
                'agency': 'obs_css_agency',
                'basePath': 'obs_path',
            },
            'tags': [
                {'key': 'key1', 'value': 'value1'},
                {'key': 'key2', 'value': 'value2'},
            ],
        }
        self.client.create_cluster.assert_called_with(**attrs)
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout, print_status=True
        )
        self.client.get_cluster.assert_called_with(self._data.id)
        self.assertEqual(self.columns, columns)


class TestRestartCluster(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 300

    def setUp(self):
        super(TestRestartCluster, self).setUp()

        self.cmd = cluster.RestartCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.restart_cluster = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_restart(self):
        arglist = [
            self._data.name,
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.restart_cluster.assert_called_with(self._data)
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestUpdateClusterName(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    columns = COLUMNS

    data = fakes.gen_data(_data, columns, cluster._formatters)

    def setUp(self):
        super(TestUpdateClusterName, self).setUp()

        self.cmd = cluster.UpdateClusterName(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.get_cluster = mock.Mock(return_value=self._data)
        self.client.update_cluster_name = mock.Mock(return_value=None)

    def test_update_name(self):
        new_name = 'new_cluster_name'
        arglist = [
            self._data.name,
            '--new-name',
            new_name,
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('new_name', new_name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.update_cluster_name.assert_called_with(
            self._data, new_name
        )
        self.client.get_cluster.assert_called_with(self._data)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateClusterPassword(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestUpdateClusterPassword, self).setUp()

        self.cmd = cluster.UpdateClusterPassword(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.update_cluster_password = mock.Mock(return_value=None)

    def test_update_password(self):
        new_password = 'new_cluster_password'
        arglist = [
            self._data.name,
            '--new-password',
            new_password,
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('new_password', new_password),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.update_cluster_password.assert_called_with(
            self._data, new_password
        )
        self.assertIsNone(result)


class TestUpdateSecurityGroup(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestUpdateSecurityGroup, self).setUp()

        self.cmd = cluster.UpdateClusterSecurityGroup(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)

        self.app.client_manager.network = mock.Mock()

        self.app.client_manager.network.find_security_group = mock.Mock(
            return_value=self._data
        )

        self.client.update_cluster_security_group = mock.Mock(
            return_value=None
        )

    def test_update_security_group(self):
        new_security_group = 'new_security_group'
        arglist = [
            self._data.name,
            '--security-group',
            new_security_group,
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('security_group', new_security_group),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )

        self.app.client_manager.network.find_security_group.assert_called_with(
            new_security_group, ignore_missing=False
        )
        self.client.update_cluster_security_group.assert_called_with(
            self._data, self._data.id
        )
        self.assertIsNone(result)

    def test_missing_security_group(self):
        arglist = [
            self._data.name,
        ]
        verifylist = [('cluster', self._data.name)]

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        ex = self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )
        self.assertIn('arguments are required: --security-group', str(ex))


class TestUpdateSecurityMode(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestUpdateSecurityMode, self).setUp()

        self.cmd = cluster.UpdateClusterSecurityMode(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)

        self.client.update_cluster_security_mode = mock.Mock(return_value=None)

    def test_update_security_mode(self):
        authority_enable = True
        https_enable = True
        arglist = [
            self._data.name,
            '--authority-enable',
            '--https-enable',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('authority_enable', authority_enable),
            ('https_enable', https_enable),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )

        self.client.update_cluster_security_mode.assert_called_with(
            self._data,
            authority_enable=authority_enable,
            https_enable=https_enable,
        )
        self.assertIsNone(result)


class TestExtendCluster(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestExtendCluster, self).setUp()

        self.cmd = cluster.ExtendCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.extend_cluster = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_extend(self):
        arglist = [
            self._data.name,
            '--add-nodes',
            '2',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('add_nodes', 2),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.extend_cluster.assert_called_with(self._data, 2)
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestExtendClusterNodes(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestExtendClusterNodes, self).setUp()

        self.cmd = cluster.ExtendClusterNodes(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.extend_cluster_nodes = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_extend(self):
        arglist = [
            self._data.name,
            '--extend',
            'type=ess,disksize=60,nodesize=2',
            '--extend',
            'type=ess-master,disksize=0,nodesize=2',
            '--wait',
        ]

        expected_request_data = {
            'grow': [
                {'type': 'ess', 'disksize': '60', 'nodesize': '2'},
                {'type': 'ess-master', 'disksize': '0', 'nodesize': '2'},
            ]
        }

        verifylist = [
            ('cluster', self._data.name),
            ('extend', expected_request_data['grow']),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.extend_cluster_nodes.assert_called_with(
            self._data, **expected_request_data
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )

        self.assertIsNone(result)


class TestShowCluster(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    columns = COLUMNS

    data = fakes.gen_data(_data, columns, cluster._formatters)

    def setUp(self):
        super(TestShowCluster, self).setUp()

        self.cmd = cluster.ShowCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('cluster', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.id, ignore_missing=False
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [
            'unexist_css_data',
        ]

        verifylist = [
            ('cluster', 'unexist_css_data'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_result = exceptions.CommandError('Resource Not Found')
        self.client.find_cluster = mock.Mock(side_effect=find_mock_result)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('Resource Not Found', str(e))
        self.client.find_cluster.assert_called_with(
            'unexist_css_data', ignore_missing=False
        )


class TestListClusterNodes(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    column_list_headers = (
        'ID',
        'Name',
        'IP',
        'Type',
        'Volume',
        'Availability Zone',
        'Status',
    )

    data = []

    for s in _data.nodes:
        data.append(
            (
                s.id,
                s.name,
                s.ip,
                s.type,
                cli_utils.YamlFormat(s.volume),
                s.availability_zone,
                s.status,
            )
        )

    def setUp(self):
        super(TestListClusterNodes, self).setUp()

        self.cmd = cluster.ListClusterNodes(self.app, None)

        self.client.find_cluster = mock.Mock()

    def test_list(self):
        arglist = [
            self._data.id,
        ]

        verifylist = [
            ('cluster', self._data.id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_cluster.side_effect = [self._data]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster.assert_called_with(
            self._data.id, ignore_missing=False
        )

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestDeleteCluster(fakes.TestCss):
    _data = fakes.FakeCluster.create_multiple(2)

    def setUp(self):
        super(TestDeleteCluster, self).setUp()

        self.client.delete_cluster = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = cluster.DeleteCluster(self.app, None)

    def test_delete(self):
        arglist = [
            self._data[0].name,
        ]

        verifylist = [
            ('cluster', [self._data[0].name]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.find_cluster = mock.Mock(return_value=self._data[0])

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_cluster.assert_called_with(
            self._data[0], ignore_missing=False
        )
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = []

        for css_data in self._data:
            arglist.append(css_data.name)

        verifylist = [
            ('cluster', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.find_cluster = mock.Mock(side_effect=self._data)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        calls = []
        for css_data in self._data:
            calls.append(call(css_data, ignore_missing=False))
        self.client.delete_cluster.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            self._data[0].name,
            'unexist_css_data',
        ]
        verifylist = [
            ('cluster', arglist),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        find_mock_results = [self._data[0], exceptions.CommandError]
        self.client.find_cluster = mock.Mock(side_effect=find_mock_results)

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual('1 of 2 Cluster(s) failed to delete.', str(e))

        self.client.find_cluster.assert_any_call(
            self._data[0].name, ignore_missing=False
        )
        self.client.find_cluster.assert_any_call(
            'unexist_css_data', ignore_missing=False
        )
        self.client.delete_cluster.assert_called_once_with(
            self._data[0], ignore_missing=False
        )


class TestUpdateClusterFlavor(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestUpdateClusterFlavor, self).setUp()

        self.cmd = cluster.UpdateClusterFlavor(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.update_cluster_flavor = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_update_flavor(self):
        new_flavor = 'new_flavor'
        arglist = [self._data.name, '--flavor', new_flavor, '--wait']

        verifylist = [
            ('cluster', self._data.name),
            ('flavor', new_flavor),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.update_cluster_flavor.assert_called_with(
            self._data, new_flavor=new_flavor
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)

    def test_update_flavor_with_node_type(self):
        node_type = 'ess'
        new_flavor = 'new_flavor'
        arglist = [
            self._data.name,
            '--node-type',
            node_type,
            '--flavor',
            new_flavor,
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('node_type', node_type),
            ('flavor', new_flavor),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.update_cluster_flavor.assert_called_with(
            self._data, node_type=node_type, new_flavor=new_flavor
        )
        self.assertIsNone(result)


class TestScaleInCluster(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestScaleInCluster, self).setUp()

        self.cmd = cluster.ScaleInCluster(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.scale_in_cluster = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_scale_in(self):
        arglist = [
            self._data.name,
            '--nodes',
            'id1',
            'id2',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('nodes', ['id1', 'id2']),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.scale_in_cluster.assert_called_with(
            self._data, ['id1', 'id2']
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestScaleInClusterByNodeType(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestScaleInClusterByNodeType, self).setUp()

        self.cmd = cluster.ScaleInClusterByNodeType(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.scale_in_cluster_by_node_type = mock.Mock(
            return_value=None
        )
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_scale_in_ess(self):
        arglist = [
            self._data.name,
            '--ess',
            '2',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('ess', 2),
            ('wait', True),
        ]

        expected_data = [{'type': 'ess', 'reducedNodeNum': 2}]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.scale_in_cluster_by_node_type.assert_called_with(
            self._data, expected_data
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)

    def test_scale_in_all_type(self):
        arglist = [
            self._data.name,
            '--ess',
            '2',
            '--ess-master',
            '3',
            '--ess-client',
            '1',
            '--ess-cold',
            '5',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('ess', 2),
            ('ess-master', 3),
            ('ess-client', 1),
            ('ess-cold', 5),
            ('wait', True),
        ]

        expected_data = [
            {'type': 'ess', 'reducedNodeNum': 2},
            {'type': 'ess-master', 'reducedNodeNum': 3},
            {'type': 'ess-client', 'reducedNodeNum': 1},
            {'type': 'ess-cold', 'reducedNodeNum': 5},
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.scale_in_cluster_by_node_type.assert_called_with(
            self._data, expected_data
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestRetryClusterUpgradeJob(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestRetryClusterUpgradeJob, self).setUp()

        self.cmd = cluster.RetryClusterUpgradeJob(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.retry_cluster_upgrade_task = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_retry_mode(self):
        arglist = [self._data.name, '--job-id', 'job_id', '--wait']

        verifylist = [
            ('cluster', self._data.name),
            ('job_id', 'job_id'),
            ('retry_mode', 'abort'),
            ('wait', True),
        ]

        expected_data = {'job_id': 'job_id', 'retry_mode': 'abort'}

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.retry_cluster_upgrade_job.assert_called_with(
            self._data, **expected_data
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestUpdateClusterKernel(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    def setUp(self):
        super(TestUpdateClusterKernel, self).setUp()

        self.cmd = cluster.UpdateClusterKernel(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.update_cluster_kernel = mock.Mock(return_value=None)

    def test_upgrade_cluster_all_checks(self):
        arglist = [
            self._data.name,
            '--target-image-id',
            'target_image_id',
            '--upgrade-type',
            'cross',
            '--agency',
            'css_agency',
            '--check-backup-indices',
            '--check-cluster-load',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('target_image_id', 'target_image_id'),
            ('upgrade_type', 'cross'),
            ('agency', 'css_agency'),
            ('check_backup_indices', True),
            ('check_cluster_load', True),
        ]

        expected_data = {
            'target_image_id': 'target_image_id',
            'upgrade_type': 'cross',
            'agency': 'css_agency',
            'indices_backup_check': True,
            'cluster_load_check': True,
        }

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.update_cluster_kernel.assert_called_with(
            self._data, **expected_data
        )
        self.assertIsNone(result)


class TestReplaceClusterNode(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestReplaceClusterNode, self).setUp()

        self.cmd = cluster.ReplaceClusterNode(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.replace_cluster_node = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_replace_node(self):
        arglist = [
            self._data.name,
            '--node-id',
            'id1',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('node_id', 'id1'),
            ('wait', True),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.replace_cluster_node.assert_called_with(self._data, 'id1')
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestAddClusterNodes(fakes.TestCss):
    _data = fakes.FakeCluster.create_one()

    default_timeout = 1200

    def setUp(self):
        super(TestAddClusterNodes, self).setUp()

        self.cmd = cluster.AddClusterNodes(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._data)
        self.client.add_cluster_nodes = mock.Mock(return_value=None)
        self.client.wait_for_cluster = mock.Mock(return_value=True)

    def test_add_nodes(self):
        arglist = [
            self._data.name,
            '--node-type',
            'ess',
            '--flavor',
            'node_flavor',
            '--node-size',
            '3',
            '--volume-type',
            'COMMON',
            '--wait',
        ]

        verifylist = [
            ('cluster', self._data.name),
            ('node_type', 'ess'),
            ('flavor', 'node_flavor'),
            ('node_size', 3),
            ('volume_type', 'COMMON'),
            ('wait', True),
        ]

        expected_data = {
            'node_type': 'ess',
            'flavor': 'node_flavor',
            'node_size': 3,
            'volume_type': 'COMMON',
        }

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.find_cluster.assert_called_with(
            self._data.name, ignore_missing=False
        )
        self.client.add_cluster_nodes.assert_called_with(
            self._data, **expected_data
        )
        self.client.wait_for_cluster.assert_called_with(
            self._data.id, self.default_timeout
        )
        self.assertIsNone(result)


class TestListClusterVersionUpgrades(fakes.TestCss):
    _data = fakes.FakeClusterImage.create_one()
    _cluster = fakes.FakeCluster.create_one()

    column_list_headers = [
        'Datastore Type',
        'Datastore Version',
        'Image Name',
        'Id',
        'Description',
        'Priority',
    ]

    columns = (
        'datastore_type',
        'datastore_version',
        'dsiplay_name',
        'id',
        'image_desc',
        'priority',
    )

    data = []

    for s in _data.image_info_list:
        data.append(
            (
                s.datastore_type,
                s.datastore_version,
                s.display_name,
                s.id,
                s.image_desc,
                s.priority,
            )
        )

    def setUp(self):
        super(TestListClusterVersionUpgrades, self).setUp()

        self.client.find_cluster = mock.Mock(return_value=self._cluster)

        self.cmd = cluster.ListClusterVersionUpgrades(self.app, None)

        self.client.clusters = mock.Mock()

    def test_list_version_upgrades(self):
        arglist = [self._cluster.name, '--upgrade-type', 'same']

        verifylist = [
            ('cluster', self._cluster.name),
            ('upgrade_type', 'same'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_cluster_version_upgrades.side_effect = [self._data]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_cluster_version_upgrades.assert_called_with(
            self._cluster, 'same'
        )

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestShowClusterUpgradeStatus(fakes.TestCss):
    _data = fakes.FakeClusterUpgradeStatus.create_multiple(3)
    _cluster = fakes.FakeCluster.create_one()

    column_list_headers = (
        'ID',
        'Image Info',
        'Execute Times',
        'Start Time',
        'End Time',
        'Status',
    )

    columns = (
        'id',
        'image_info',
        'execute_times',
        'start_time',
        'end_time',
        'status',
    )

    data = []

    for s in _data:
        data.append(
            (
                s.id,
                cli_utils.YamlFormat(s.image_info),
                s.execute_times,
                s.start_time,
                s.end_time,
                s.status,
            )
        )

    def setUp(self):
        super(TestShowClusterUpgradeStatus, self).setUp()

        self.cmd = cluster.ShowClusterUpgradeStatus(self.app, None)

        self.client.find_cluster = mock.Mock(return_value=self._cluster)

        self.client.get_cluster_upgrade_status = mock.Mock()

    def test_status_show(self):
        arglist = [self._cluster.name]

        verifylist = [
            ('cluster', self._cluster.name),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_cluster_upgrade_status.side_effect = [self._data]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_cluster_upgrade_status.assert_called_with(
            self._cluster
        )

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource


class TagsSpec(resource.Resource):
    #: Specifies the tag key.
    key = resource.Body('key')
    #: Specifies the tag value.
    value = resource.Body('value')


class Endpoint(resource.Resource):
    base_path = '/vpc-endpoints'
    resources_key = 'endpoints'
    resource_key = None

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'id',
        'offset',
        'limit',
        'router_id',
        'endpoint_service_name',
        'sort_dir',
        'sort_key',
        router_id='vpc_id',
    )

    # Properties
    #: Domain status.
    active_status = resource.Body('active_status', type=list)
    #: Creation time of the VPC endpoint.
    created_at = resource.Body('created_at')
    #: Description of the VPC endpoint.
    description = resource.Body('description')
    #: Domain name for accessing the associated VPC endpoint service.
    dns_names = resource.Body('dns_names', type=list)
    #: ID of the cluster associated with the VPC endpoint.
    endpoint_pool_id = resource.Body('endpoint_pool_id')
    #: ID of the VPC endpoint service.
    endpoint_service_id = resource.Body('endpoint_service_id')
    #: Name of the VPC endpoint service.
    endpoint_service_name = resource.Body('endpoint_service_name')
    #: VPC endpoint ID.
    id = resource.Body('id')
    #: IP address for accessing the associated VPC endpoint service.
    ip = resource.Body('ip')
    #: IP address for accessing the associated VPC endpoint service.
    port_ip = resource.Body('port_ip', alias='ip')
    #: Whether to create a private domain name.
    is_dns_enabled = resource.Body('enable_dns', type=bool)
    #: Whether the VPC endpoint is enabled.
    is_enabled = resource.Body('enable_status', type=bool)
    #: Whether access control is enabled.
    is_whitelist_enabled = resource.Body('enable_whitelist', type=bool)
    #: Packet ID of the VPC endpoint.
    marker_id = resource.Body('marker_id', type=int)
    #: Specifies the ID of the network in the VPC/Router.
    network_id = resource.Body('subnet_id')
    #: Project ID.
    project_id = resource.Body('project_id')
    #: IDs of route tables.
    route_tables = resource.Body('routetables', type=list)
    #: ID of the VPC/Router where the VPC endpoint is to be created.
    router_id = resource.Body('vpc_id')
    #: Type of the VPC endpoint service that is associated with the
    #:  VPC endpoint.
    service_type = resource.Body('service_type')
    #: Specifies the name of the VPC endpoint specifications.
    specification_name = resource.Body('specification_name')
    #: Specifies the connection status of the VPC endpoint.
    status = resource.Body('status')
    #: Lists the resource tags.
    tags = resource.Body('tags', type=list, list_type=TagsSpec)
    #: Update time of the VPC endpoint.
    updated_at = resource.Body('updated_at')
    #: Whitelist for controlling access to the VPC endpoint.
    whitelist = resource.Body('whitelist', type=list)

    @classmethod
    def existing(cls, connection=None, **kwargs):
        """Create an instance of an existing remote resource.

        When creating the instance set the ``_synchronized`` parameter
        of :class:`Resource` to ``True`` to indicate that it represents the
        state of an existing server-side resource. As such, all attributes
        passed in ``**kwargs`` are considered "clean", such that an immediate
        :meth:`update` call would not generate a body of attributes to be
        modified on the server.

        :param dict kwargs: Each of the named arguments will be set as
            attributes on the resulting Resource object.
        """
        if 'enable_status' in kwargs.keys():
            enable_status = kwargs['enable_status']
            kwargs['enable_status'] = (
                True if enable_status == 'enable' else False
            )
        return cls(_synchronized=True, connection=connection, **kwargs)

    def _translate_response(
        self,
        response,
        has_body=None,
        error_message=None,
        *,
        resource_response_key=None,
    ):
        """Given a KSA response, inflate this instance with its data

        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.

        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body

        exceptions.raise_from_response(response, error_message=error_message)

        if has_body:
            try:
                body = response.json()
                if resource_response_key and resource_response_key in body:
                    body = body[resource_response_key]
                elif self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]
                if 'enable_status' in body.keys():
                    enable_status = body['enable_status']
                    body['enable_status'] = (
                        True if enable_status == 'enable' else False
                    )
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                body.pop("self", None)

                body_attrs = self._consume_body_attrs(body)
                if self._allow_unknown_attrs_in_body:
                    body_attrs.update(body)
                    self._unknown_attrs_in_body.update(body)
                elif self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body
                    )

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import function


EXAMPLE = {
    'func_name': 'test-function',
    'package': 'default',
    'runtime': 'Python3.9',
    'handler': 'index.handler',
    'timeout': 30,
    'memory_size': 128,
    'code_type': 'inline',
}


class TestFunction(base.TestCase):

    def test_basic(self):
        sot = function.Function()
        self.assertEqual('functions', sot.resources_key)
        path = '/fgs/functions'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = function.Function(**EXAMPLE)
        self.assertEqual(EXAMPLE['func_name'], sot.func_name)
        self.assertEqual(EXAMPLE['package'], sot.package)
        self.assertEqual(EXAMPLE['runtime'], sot.runtime)
        self.assertEqual(EXAMPLE['handler'], sot.handler)
        self.assertEqual(EXAMPLE['timeout'], sot.timeout)
        self.assertEqual(EXAMPLE['memory_size'], sot.memory_size)
        self.assertEqual(EXAMPLE['code_type'], sot.code_type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.sdrs.v1 import protection_group as _protection_group


EXAMPLE = {
    'id': uuid.uuid4(),
    'name': uuid.uuid4(),
    'description': uuid.uuid4(),
    'source_availability_zone': uuid.uuid4(),
    'target_availability_zone': uuid.uuid4(),
    'domain_id': uuid.uuid4(),
    'domain_name': uuid.uuid4(),
    'status': 'available',
    'protected_status': None,
    'replication_status': None,
    'health_status': None,
    'progress': 0,
    'priority_station': 'source',
    'protected_instance_num': 0,
    'replication_num': 0,
    'disaster_recovery_drill_num': 0,
    'source_vpc_id': uuid.uuid4(),
    'target_vpc_id': uuid.uuid4(),
    'test_vpc_id': None,
    'dr_type': 'migration',
    'created_at': '2022-02-26 20:39:30.718',
    'updated_at': '2022-02-26 21:02:45.507',
    'protection_type': 'replication-pair',
    'replication_model': None
}


class TestProtectionGroup(base.TestCase):

    def setUp(self):
        super(TestProtectionGroup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _protection_group.ProtectionGroup()

    def test_basic(self):
        sot = _protection_group.ProtectionGroup()
        self.assertEqual('server_group', sot.resource_key)
        self.assertEqual('server_groups', sot.resources_key)
        self.assertEqual('/server-groups',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertDictEqual({
            'availability_zone': 'availability_zone',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'offset': 'offset',
            'query_type': 'query_type',
            'status': 'status'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        test_protection_group = _protection_group.ProtectionGroup(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['id'],
            test_protection_group.id)
        self.assertEqual(
            EXAMPLE['description'],
            test_protection_group.description)
        self.assertEqual(
            EXAMPLE['source_availability_zone'],
            test_protection_group.source_availability_zone)
        self.assertEqual(
            EXAMPLE['target_availability_zone'],
            test_protection_group.target_availability_zone)
        self.assertEqual(
            EXAMPLE['domain_id'],
            test_protection_group.domain_id)
        self.assertEqual(
            EXAMPLE['domain_name'],
            test_protection_group.domain_name)
        self.assertEqual(
            EXAMPLE['status'],
            test_protection_group.status)
        self.assertEqual(
            EXAMPLE['protected_status'],
            test_protection_group.protected_status)
        self.assertEqual(
            EXAMPLE['replication_status'],
            test_protection_group.replication_status)
        self.assertEqual(
            EXAMPLE['health_status'],
            test_protection_group.health_status)
        self.assertEqual(
            EXAMPLE['progress'],
            test_protection_group.progress)
        self.assertEqual(
            EXAMPLE['priority_station'],
            test_protection_group.priority_station)
        self.assertEqual(
            EXAMPLE['protected_instance_num'],
            test_protection_group.protected_instance_num)
        self.assertEqual(
            EXAMPLE['replication_num'],
            test_protection_group.replication_num)
        self.assertEqual(
            EXAMPLE['disaster_recovery_drill_num'],
            test_protection_group.disaster_recovery_drill_num)
        self.assertEqual(
            EXAMPLE['source_vpc_id'],
            test_protection_group.source_vpc_id)
        self.assertEqual(
            EXAMPLE['target_vpc_id'],
            test_protection_group.target_vpc_id)
        self.assertEqual(
            EXAMPLE['test_vpc_id'],
            test_protection_group.test_vpc_id)
        self.assertEqual(
            EXAMPLE['dr_type'],
            test_protection_group.dr_type)
        self.assertEqual(
            EXAMPLE['created_at'],
            test_protection_group.created_at)
        self.assertEqual(
            EXAMPLE['updated_at'],
            test_protection_group.updated_at)
        self.assertEqual(
            EXAMPLE['protection_type'],
            test_protection_group.protection_type)
        self.assertEqual(
            EXAMPLE['replication_model'],
            test_protection_group.replication_model)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

'''MRS clusters v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)

_formatters = {}


def _get_columns(item):
    column_map = {}
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListDatasource(command.Lister):
    _description = _('List Sahara datasoruces')
    columns = (
        'id', 'name', 'type', 'url', 'description',
        'is_public', 'is_protected'
    )

    def get_parser(self, prog_name):
        parser = super(ListDatasource, self).get_parser(prog_name)
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_('type of the datasource')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Number of entries to display.')
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('ID of the last record on the previous page.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.mrs

        query = {}

        if parsed_args.type:
            query['type'] = parsed_args.type
        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.marker:
            query['marker'] = parsed_args.marker

        data = client.datasources(**query)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table


class DeleteDatasource(command.Command):
    _description = _('Delete Data Source')

    def get_parser(self, prog_name):
        parser = super(DeleteDatasource, self).get_parser(prog_name)

        parser.add_argument(
            'id',
            metavar='<id>',
            nargs='+',
            help=_('Data source ID')
        )

        return parser

    def take_action(self, parsed_args):
        if parsed_args.id:
            client = self.app.client_manager.mrs
            for id in parsed_args.id:
                client.delete_datasource(id, ignore_missing=False)


class CreateDatasource(command.ShowOne):
    _description = _('Create datasource')

    columns = ('id', 'name', 'type', 'url', 'description',
               'is_public', 'is_protected')

    def get_parser(self, prog_name):
        parser = super(CreateDatasource, self).get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help=_('Name for the datasource')
        )
        parser.add_argument(
            '--is_public',
            action='store_const',
            default='false',
            const='false',
            dest='is_public',
            help=_('Whether the data source is public')
        )
        parser.add_argument(
            '--is_protected',
            action='store_const',
            default='false',
            const='false',
            dest='is_protected',
            help=_('Whether the data source is protected')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            required=True,
            help=_('Data source type')
        )
        parser.add_argument(
            '--url',
            metavar='<url>',
            required=True,
            help=_('Data source URL')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Data source description')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.mrs

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.type:
            attrs['type'] = parsed_args.type
        if parsed_args.url:
            attrs['url'] = parsed_args.url
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.is_public:
            attrs['is_public'] = parsed_args.is_public
        if parsed_args.is_protected:
            attrs['is_protected'] = parsed_args.is_protected
        obj = client.create_datasource(
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return display_columns, data


class UpdateDatasource(command.ShowOne):
    _description = _('Update Data Source')

    columns = ('id', 'name', 'type', 'url', 'description',
               'is_public', 'is_protected')

    def get_parser(self, prog_name):
        parser = super(UpdateDatasource, self).get_parser(prog_name)

        parser.add_argument(
            'datasource',
            metavar='<datasource>',
            help=_('ID or name of the data source')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name for the data source')
        )
        parser.add_argument(
            '--is_public',
            action='store_const',
            default='false',
            const='false',
            dest='is_public',
            help=_('Whether the data source is public')
        )
        parser.add_argument(
            '--is_protected',
            action='store_const',
            const='false',
            dest='is_protected',
            help=_('Whether the data source is protected')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_('Data source type')
        )
        parser.add_argument(
            '--url',
            metavar='<url>',
            help=_('Data source URL')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Data source description')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.mrs

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.type:
            attrs['type'] = parsed_args.type
        if parsed_args.url:
            attrs['url'] = parsed_args.url
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.is_public:
            attrs['is_public'] = parsed_args.is_public
        if parsed_args.is_protected:
            attrs['is_protected'] = parsed_args.is_protected

        datasource = client.find_datasource(
            parsed_args.datasource,
            ignore_missing=False
        )
        if attrs:
            obj = client.update_datasource(
                datasource=datasource,
                **attrs
            )
        else:
            obj = datasource
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return display_columns, data


class ShowDatasource(command.ShowOne):
    _description = _('Show the MRS Data source details')

    def get_parser(self, prog_name):
        parser = super(ShowDatasource, self).get_parser(prog_name)

        parser.add_argument(
            'datasource',
            metavar='<datasource>',
            help=_('ID or name of the of the data source')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.mrs

        obj = client.find_datasource(
            parsed_args.datasource,
            ignore_missing=False
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return display_columns, data



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from openstack import exceptions

from otcextensions.tests.unit.sdk import base


class TestDdsMixin(base.TestCase):

    def setUp(self):
        super(TestDdsMixin, self).setUp()

    def test_create_dds_instance(self):
        attrs = {
            'name': 'dds_name',
            'datastore_type': 'DDS',
            'datastore_version': '3.1',
            'datastore_storage_engine': 'wt',
            'region': 'eu-de',
            'availability_zone': 'az1',
            'router': 'my_router',
            'network': 'my_network',
            'security_group': 'my_security_group',
            'password': 'password12344@@!',
            'mode': 'ReplicaSet',
            'flavors': [{
                "type": "replica",
                "num": 1,
                "storage": "ULTRAHIGH",
                "size": 30,
                "spec_code": "dds.mongodb.s2.medium.4.repset"
            }],
            'backup_timeframe': '23:00-00:00',
            'backup_keepdays': '1',
            'ssl_option': '0',
        }

        self.register_uris([
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['my_router']
                ),
                json={'id': 'router_id'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['my_network']
                ),
                json={'id': 'net_id'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='security-groups',
                    base_url_append='v2.0',
                    append=['my_security_group']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_dds_url(
                    resource='flavors',
                    qs_elements=[f'region={attrs["region"]}',
                                 f'engine_name={attrs["datastore_type"]}']
                ),
                status_code=200,
                json={'flavors': [
                    {"type": "replica",
                     "num": 1,
                     "storage": "ULTRAHIGH",
                     "size": 30,
                     "spec_code": "dds.mongodb.s2.medium.4.repset"
                     }]
                }
            ),
            dict(
                method='POST',
                uri=self.get_dds_url(
                    base_url_append='instances'
                ),
                status_code=200,
                json={
                    'instance': {'id': 123987},
                    'job_id': '15'
                }
            ),
            dict(
                method='GET',
                uri=self.get_dds_url(
                    base_url_append='instances',
                    qs_elements=['id=123987']
                ),
                status_code=200,
                json={'instances': [{'id': 123987, 'name': 'inst_name'}]}
            )
        ])

        obj = self.cloud.create_dds_instance(**attrs)
        self.assert_calls()

        self.assertEqual(123987, obj.id)

    def test_create_dds_instance_bad_flavor_spec_code(self):
        attrs = {
            'name': 'dds_name',
            'datastore_type': 'DDS',
            'datastore_version': '3.1',
            'datastore_storage_engine': 'wt',
            'region': 'eu-de',
            'availability_zone': 'az1',
            'router': 'my_router',
            'network': 'my_network',
            'security_group': 'my_security_group',
            'password': 'password12344@@!',
            'mode': 'ReplicaSet',
            'flavors': [{
                "type": "replica",
                "num": 1,
                "storage": "ULTRAHIGH",
                "size": 30,
                "spec_code": "dds"
            }],
            'backup_timeframe': '23:00-00:00',
            'backup_keepdays': '1',
            'ssl_option': '0',
        }

        self.register_uris([
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='routers',
                    base_url_append='v2.0',
                    append=['my_router']
                ),
                json={'id': 'router_id'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='networks',
                    base_url_append='v2.0',
                    append=['my_network']
                ),
                json={'id': 'net_id'}
            ),
            dict(
                method='GET',
                uri=self.get_mock_url(
                    'network',
                    resource='security-groups',
                    base_url_append='v2.0',
                    append=['my_security_group']
                ),
                json={'id': 'fake'}
            ),
            dict(
                method='GET',
                uri=self.get_dds_url(
                    resource='flavors',
                    qs_elements=[f'region={attrs["region"]}',
                                 f'engine_name={attrs["datastore_type"]}']
                ),
                status_code=200,
                json={'flavors': [
                    {"type": "replica",
                     "num": 1,
                     "storage": "ULTRAHIGH",
                     "size": 30,
                     "spec_code": "dds.mongodb.s2.medium.4.repset"
                     }]
                }
            )
        ])

        self.assertRaises(
            exceptions.SDKException,
            self.cloud.create_dds_instance,
            **attrs
        )
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from urllib.parse import urlparse

from openstack.utils import urljoin

from openstack.identity.v3 import _proxy

from otcextensions.sdk.identity.v3 import agency as _agency
from otcextensions.sdk.identity.v3 import agency_role as _agency_role
from otcextensions.sdk.identity.v3 import credential as _credential
from otcextensions.sdk.identity.v3 import security_token as _security_token
from otcextensions.sdk.identity.v3 import custom_role as _custom


class Proxy(_proxy.Proxy):

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self._credentials_base = None

    def _get_alternate_endpoint(self):
        if not self._credentials_base:
            identity_url = self.get_endpoint_data().url
            parsed_domain = urlparse(identity_url)
            self._credentials_base = '%s://%s' % (parsed_domain.scheme,
                                                  parsed_domain.netloc)
        return self._credentials_base

    # ========== Credentials ==========
    def credentials(self, **attrs):
        """Retrieve a generator of credentials

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `user_id`: user_id

        :returns: A generator of credentials
            :class:`~otcextensions.sdk.identity.v3.credential.Credential`
            instances
        """
        # for list we need to pass corrected endpoint
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _credential.Credential.base_path)
        return self._list(_credential.Credential, base_path=base_path,
                          **attrs)

    def create_credential(self, **attrs):
        """Create a new credential from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.identity.v3.credential.Credential`,
            comprised of the properties on the Credential class.
        :returns: The results of credential creation
        :rtype: :class:`~otcextensions.sdk.identity.v3.credential.Credential`
        """
        return self._create(_credential.Credential, prepend_key=True,
                            **attrs)

    def get_credential(self, credential):
        """Get a credential

        :param credential: The value can be the ID of a credential
            or a :class:`~otcextensions.sdk.identity.v3.credential.Credential`
            instance.
        :returns: Credential instance
        :rtype: :class:`~otcextensions.sdk.identity.v3.credential.Credential`
        """
        return self._get(_credential.Credential, credential)

    def find_credential(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single credential

        :param name_or_id: The name or ID of a credential
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the credential does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent credential.

        :returns: ``None``
        """
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _credential.Credential.base_path)
        return self._find(_credential.Credential, name_or_id,
                          ignore_missing=ignore_missing,
                          base_path=base_path,
                          **attrs)

    def delete_credential(self, credential, ignore_missing=True):
        """Delete a credential

        :param credential: The value can be the ID of a credential
            or a :class:`~otcextensions.sdk.identity.v3.credential.Credential`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the credential does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent credential.

        :returns: Credential been deleted
        :rtype: :class:`~otcextensions.sdk.identity.v3.credential.Credential`
        """
        return self._delete(_credential.Credential,
                            credential,
                            ignore_missing=ignore_missing)

    def update_credential(self, credential, **attrs):
        """Update credential attributes

        :param credential: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.credential.Credential`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.identity.v3.credential.Credential`

        :rtype: :class:`~otcextensions.sdk.identity.v3.credential.Credential`
        """
        return self._update(_credential.Credential, credential, **attrs)

    # ========== Agencies ==========
    def _agency_normalize_domain_id(self, **attrs):
        if 'domain_id' not in attrs:
            # User missed passing domain_id. Let's assume he wants to list own
            # agencies.
            # Otherwise he should have passed domain_id=None
            access = self.session.auth.get_auth_ref(self)
            if access.domain_id:
                attrs['domain_id'] = access.domain_id
            elif access.project_domain_id:
                attrs['domain_id'] = access.project_domain_id
            elif access.user_domain_id:
                attrs['domain_id'] = access.user_domain_id
        elif attrs['domain_id'] is None:
            # even though requests will eject this prop, do not rely on this
            # functionality
            attrs.pop('domain_id')
        return attrs

    def agencies(self, **attrs):
        """Retrieve a generator of agencies

        When domain_id query parameter is not set - current domain_id will be
        used. Passing domain_id=None allow removing filtering.

        :param dict attrs: Optional query parameters to be sent to limit the
            resources being returned.
            * `domain_id`: Current domain ID
            * `name`: Name of the agency
            * `trust_domain_id`: ID of the delegated domain.

        :returns: A generator of agencies
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
            instances
        """
        # for list we need to pass corrected endpoint
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _agency.Agency.base_path)
        attrs = self._agency_normalize_domain_id(**attrs)

        return self._list(_agency.Agency, base_path=base_path,
                          **attrs)

    def create_agency(self, **attrs):
        """Create a new agency from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.identity.v3.agency.Agency`,
            comprised of the properties on the Agency class.
        :returns: The results of agency creation
        :rtype: :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        """
        attrs = self._agency_normalize_domain_id(**attrs)
        return self._create(_agency.Agency, prepend_key=True,
                            **attrs)

    def get_agency(self, agency):
        """Get a agency

        :param agency: The value can be the ID of a agency
            or a :class:`~otcextensions.sdk.identity.v3.agency.Agency`
            instance.
        :returns: Agency instance
        :rtype: :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        """
        return self._get(_agency.Agency, agency)

    def find_agency(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single agency

        :param name_or_id: The name or ID of a agency
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the agency does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent agency.

        :returns: ``None``
        """
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _agency.Agency.base_path)
        attrs = self._agency_normalize_domain_id(**attrs)
        return self._find(_agency.Agency, name_or_id,
                          ignore_missing=ignore_missing,
                          base_path=base_path,
                          **attrs)

    def delete_agency(self, agency, ignore_missing=True):
        """Delete a agency

        :param agency: The value can be the ID of a agency
            or a :class:`~otcextensions.sdk.identity.v3.agency.Agency`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the agency does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent agency.

        :returns: Agency been deleted
        :rtype: :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        """
        return self._delete(_agency.Agency,
                            agency,
                            ignore_missing=ignore_missing)

    def update_agency(self, agency, **attrs):
        """Update agency attributes

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`

        :rtype: :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        """
        return self._update(_agency.Agency, agency, **attrs)

    # ========== Agency roles ==========

    def agency_project_roles(self, agency, project_id):
        """Retrieve a generator of agency roles on a project

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param project_id: ID of a project

        :returns: A generator of agencies
            :class:`~otcextensions.sdk.identity.v3.agency_role.AgencyRole`
            instances
        """
        # for list we need to pass corrected endpoint
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _agency_role.AgencyRole.base_path)
        agency = self._get_resource(_agency.Agency, agency)

        return self._list(_agency_role.AgencyRole,
                          base_path=base_path,
                          role_ref_type='project',
                          role_ref_id=project_id,
                          agency_id=agency.id)

    def check_agency_project_role(self, agency, project_id, role_id):
        """Check whether role is granted on the project through agency

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param project_id: ID of a project
        :param role_id: ID of a role to check

        :returns:
            :class:`~otcextensions.sdk.identity.v3.agency_role.AgencyRole`
         """
        agency = self._get_resource(_agency.Agency, agency)
        agency_role = self._get_resource(
            _agency_role.AgencyRole,
            {
                'agency_id': agency.id,
                'role_ref_type': 'project',
                'role_ref_id': project_id,
                'id': role_id
            }
        )
        return self._head(_agency_role.AgencyRole,
                          agency_role)

    def grant_agency_project_role(self, agency, project_id, role_id):
        """Grant permission of agency on a project

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param project_id: ID of a project
        :param role_id: ID of a role to revoke

        :returns:
         """
        agency = self._get_resource(_agency.Agency, agency)
        agency_role = self._get_resource(
            _agency_role.AgencyRole,
            {
                'agency_id': agency.id,
                'role_ref_type': 'project',
                'role_ref_id': project_id,
                'id': role_id
            }
        )
        req = agency_role._prepare_request()
        self.put(req.url)

    def revoke_agency_project_role(self, agency, project_id, role_id):
        """Revoke permission of agency on a project

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param project_id: ID of a project
        :param role_id: ID of a role to revoke

        :returns:
         """
        agency = self._get_resource(_agency.Agency, agency)
        agency_role = self._get_resource(
            _agency_role.AgencyRole,
            {
                'agency_id': agency.id,
                'role_ref_type': 'project',
                'role_ref_id': project_id,
                'id': role_id
            }
        )
        return self._delete(_agency_role.AgencyRole, agency_role)

    def agency_domain_roles(self, agency, domain_id):
        """Retrieve a generator of agency roles on a domain

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param domain_id: ID of a domain

        :returns: A generator of agencies
            :class:`~otcextensions.sdk.identity.v3.agency_role.AgencyRole`
            instances
        """
        # for list we need to pass corrected endpoint
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _agency_role.AgencyRole.base_path)
        agency = self._get_resource(_agency.Agency, agency)

        return self._list(_agency_role.AgencyRole,
                          base_path=base_path,
                          role_ref_type='domain',
                          role_ref_id=domain_id,
                          agency_id=agency.id)

    def check_agency_domain_role(self, agency, domain_id, role_id):
        """Check whether role is granted on the domain through agency

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param domain_id: ID of a domain
        :param role_id: ID of a role to check

        :returns:
            :class:`~otcextensions.sdk.identity.v3.agency_role.AgencyRole`
         """
        agency = self._get_resource(_agency.Agency, agency)
        agency_role = self._get_resource(
            _agency_role.AgencyRole,
            {
                'agency_id': agency.id,
                'role_ref_type': 'domain',
                'role_ref_id': domain_id,
                'id': role_id
            }
        )
        return self._head(_agency_role.AgencyRole,
                          agency_role)

    def grant_agency_domain_role(self, agency, domain_id, role_id):
        """Grant permission of agency on a domain

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param domain_id: ID of a domain
        :param role_id: ID of a role to revoke

        :returns:
         """
        agency = self._get_resource(_agency.Agency, agency)
        agency_role = self._get_resource(
            _agency_role.AgencyRole,
            {
                'agency_id': agency.id,
                'role_ref_type': 'domain',
                'role_ref_id': domain_id,
                'id': role_id
            }
        )
        req = agency_role._prepare_request()
        self.put(req.url)

    def revoke_agency_domain_role(self, agency, domain_id, role_id):
        """Revoke permission of agency on a domain

        :param agency: The id or an instance of
            :class:`~otcextensions.sdk.identity.v3.agency.Agency`
        :param domain_id: ID of a domain
        :param role_id: ID of a role to revoke

        :returns:
         """
        agency = self._get_resource(_agency.Agency, agency)
        agency_role = self._get_resource(
            _agency_role.AgencyRole,
            {
                'agency_id': agency.id,
                'role_ref_type': 'domain',
                'role_ref_id': domain_id,
                'id': role_id
            }
        )
        return self._delete(_agency_role.AgencyRole, agency_role)

    # ========== Security Token (temp AK/SK) ==========

    def create_security_token(self, duration, method='token', **attrs):
        """Create a new temporary AK/SK

        :param int duration: Duration in seconds for the token validity.
        :param str method: Authorization method (token or agency)
        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.identity.v3.security_token.SecurityToken`,
            comprised of the properties on the SecurityToken class.
        :returns: The results of temporary security token creation
        :rtype:
            :class:`~otcextensions.sdk.identity.v3.security_token.SecurityToken`
        """
        # This method is so unique (similar to getting initial auth), that we
        # need to do this totally differently
        body = {
            'auth': {
                'identity': {
                    'methods': [method],
                    method: {**attrs}
                },
            }
        }
        body['auth']['identity'][method]['duration-secods'] = duration
        uri = '%s/v3.0/OS-CREDENTIAL/securitytokens' % (
            self._get_alternate_endpoint()
        )
        response = self.post(uri, json=body)
        token = _security_token.SecurityToken()
        token._translate_response(response)
        return token

    def custom_roles(self, **attrs):
        """Retrieve a generator of custom roles

        :param dict attrs: Optional query parameters to be sent to limit the
            resources being returned.
            * `page`: Page number for pagination query.
            * `per_page`: Number of data records to be displayed on each page.

        :returns: A generator of custom roles
            :class:`~otcextensions.sdk.identity.v3._custom.CustomRole`
            instances
        """
        # for list, we need to pass corrected endpoint
        base = self._get_alternate_endpoint()
        base_path = urljoin(base, _custom.CustomRole.base_path)

        return self._list(_custom.CustomRole, base_path=base_path,
                          **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import throttling_policy as tp

EXAMPLE = {
    "name": "throttle_demo",
    "created_at": "2020-07-31T08:44:02.205366118Z",
    "remark": "Total: 800 calls/second;"
              " user: 500 calls/second;"
              " app: 300 calls/second;"
              " IP address: 600 calls/second",
    "type": 1,
    "time_interval": 1,
    "ip_call_limits": 600,
    "app_call_limits": 300,
    "time_unit": "SECOND",
    "api_call_limits": 800,
    "id": "3437448ad06f4e0c91a224183116e965",
    "user_call_limits": 500,
    "enable_adaptive_control": "FALSE",
    "bind_num": 0,
    "is_inclu_special_throttle": 2
}


class TestThrottlingPolicy(base.TestCase):

    def test_basic(self):
        sot = tp.ThrottlingPolicy()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/throttles',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertEqual('throttles', sot.resources_key)

    def test_make_it(self):
        sot = tp.ThrottlingPolicy(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['remark'], sot.remark)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['time_interval'], sot.time_interval)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import l7_rule

EXAMPLE = {
    'compare_type': 'compare_type',
    'admin_state_up': True,
    'value': 'value',
    'type': 'type',
}


class TestLoadBalancer(base.TestCase):

    def test_basic(self):
        sot = l7_rule.L7Rule()
        path = '/elb/l7policies/%(l7policy_id)s/rules'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = l7_rule.L7Rule(**EXAMPLE)
        self.assertEqual(EXAMPLE['compare_type'], sot.compare_type)
        self.assertEqual(EXAMPLE['value'], sot.rule_value)
        self.assertEqual(EXAMPLE['type'], sot.type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.rds.v3 import configuration
from otcextensions.tests.unit.osclient.rds.v3 import fakes


class TestListConfiguration(fakes.TestRds):

    objects = fakes.FakeConfiguration.create_multiple(3)

    column_list_headers = [
        'ID',
        'Name',
        'Description',
        'Datastore Name',
        'Datastore Version Name',
        'User Defined'
    ]

    columns = ('id', 'name', 'description', 'datastore_name',
               'datastore_version_name', 'is_user_defined')

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListConfiguration, self).setUp()

        self.cmd = configuration.ListConfigurations(self.app, None)

        self.client.configurations = mock.Mock()
        self.client.api_mock = self.client.configurations

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestShowConfiguration(fakes.TestRds):

    _data = fakes.FakeConfiguration.create_one()

    columns = (
        'apply_results', 'datastore_name', 'datastore_version_name',
        'description', 'id', 'is_user_defined', 'name'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestShowConfiguration, self).setUp()

        self.cmd = configuration.ShowConfiguration(self.app, None)

        self.client.find_configuration = mock.Mock(return_value=self._data)

    def test_show(self):
        arglist = [
            'test',
        ]

        verifylist = [
            ('configuration', 'test'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.find_configuration.assert_called_with('test',
                                                          ignore_missing=False)

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestListConfigurationParameters(fakes.TestRds):

    config = fakes.FakeConfiguration.create_one()

    objects = config.configuration_parameters

    column_list_headers = (
        'Name', 'Value', 'Type', 'Description',
        'Restart Required', 'Readonly', 'Value Range'
    )

    columns = (
        'name', 'value', 'type', 'description',
        'restart_required', 'readonly', 'value_range'
    )

    data = []

    for s in objects:
        # NOTE: please fix this. For some reason comprehension doesn't work
        # here
        values = []
        for attr in columns:
            values.append(s.get(attr, ''))
        data.append(tuple(values))

    def setUp(self):
        super(TestListConfigurationParameters, self).setUp()

        self.cmd = configuration.ListConfigurationParameters(self.app, None)

        self.client.find_configuration = mock.Mock(return_value=self.config)
        self.client.api_mock = self.client.find_configuration

    def test_list(self):
        arglist = ['test_config']

        verifylist = [
            ('configuration', 'test_config')
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with('test_config',
                                                ignore_missing=False)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))


class TestCreateConfiguration(fakes.TestRds):

    _data = fakes.FakeConfiguration.create_one()

    columns = (
        'apply_results', 'datastore_name', 'datastore_version_name',
        'description', 'id', 'is_user_defined', 'name'
    )

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestCreateConfiguration, self).setUp()

        self.cmd = configuration.CreateConfiguration(self.app, None)

        self.client.create_configuration = mock.Mock()
        self.client.api_mock = self.client.create_configuration

    def test_create(self):
        arglist = [
            'cfg',
            '--datastore-type', 'postgresql',
            '--datastore-version', '-9.6',
            '--description', 'descr',
            '--value', 'a=b',
            '--value', 'c=d'
        ]

        verifylist = [
            ('name', 'cfg'),
            ('datastore_type', 'postgresql'),
            ('datastore_version', '-9.6'),
            ('description', 'descr'),
            ('values', {'a': 'b', 'c': 'd'})
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            datastore={'type': 'postgresql', 'version': '-9.6'},
            description='descr',
            name='cfg',
            values={'a': 'b', 'c': 'd'}
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_create2(self):
        arglist = [
            'cfg',
            '--datastore-type', 'MySQL',
            '--datastore-version', '-9.6',
        ]

        verifylist = [
            ('name', 'cfg'),
            ('datastore_type', 'mysql'),
            ('datastore_version', '-9.6'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            datastore={'type': 'mysql', 'version': '-9.6'},
            name='cfg'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetConfiguration(fakes.TestRds):

    def setUp(self):
        super(TestSetConfiguration, self).setUp()

        self.cmd = configuration.SetConfiguration(self.app, None)

        self.client.update_configuration = mock.Mock()
        self.client.api_mock = self.client.update_configuration
        self.client.find_configuration = mock.Mock()

    def test_action(self):
        arglist = [
            't1',
            '--name', 'new_name',
            '--description', 'new_descr',
            '--value', 'q=z',
            '--value', 'w=u'
        ]
        verifylist = [
            ('configuration', 't1'),
            ('name', 'new_name'),
            ('description', 'new_descr'),
            ('values', {'q': 'z', 'w': 'u'})
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_configuration.return_value = 't1'

        # Trigger the action
        resp = self.cmd.take_action(parsed_args)

        self.client.find_configuration.assert_called_with('t1',
                                                          ignore_missing=False)
        self.client.api_mock.assert_called_with(
            't1',
            description='new_descr',
            name='new_name',
            values={'q': 'z', 'w': 'u'}
        )
        self.assertIsNone(resp)


class TestDeleteConfiguration(fakes.TestRds):

    def setUp(self):
        super(TestDeleteConfiguration, self).setUp()

        self.cmd = configuration.DeleteConfiguration(self.app, None)

        self.client.delete_configuration = mock.Mock()
        self.client.api_mock = self.client.delete_configuration
        self.client.find_configuration = mock.Mock()

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('configuration', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [{}, {}]
        self.client.find_configuration.side_effect = ['t1', 't2']

        # Trigger the action
        self.cmd.take_action(parsed_args)

        find_calls = [
            mock.call('t1', ignore_missing=False),
            mock.call('t2', ignore_missing=False)
        ]

        delete_calls = [
            mock.call('t1'),
            mock.call('t2')
        ]

        self.client.find_configuration.assert_has_calls(find_calls)
        self.client.api_mock.assert_has_calls(delete_calls)


class TestApplyConfiguration(fakes.TestRds):

    _data = fakes.FakeConfiguration.create_one()

    columns = (
        'instance_id', 'instance_name',
        'restart_required', 'success'
    )

    column_headers = (
        'ID', 'Name', 'Restart required', 'success'
    )

    data = []
    for s in _data.apply_results:
        values = []
        for attr in columns:
            values.append(s.get(attr, ''))
        data.append(tuple(values))

    def setUp(self):
        super(TestApplyConfiguration, self).setUp()

        self.cmd = configuration.ApplyConfiguration(self.app, None)

        self.client.find_configuration = mock.Mock()
        self.client.apply_configuration = mock.Mock(return_value=self._data)
        self.client.get_instance = mock.Mock()

    def test_apply(self):
        arglist = [
            'cfg',
            '--instance', 'i1',
            '--instance', 'i2',
        ]

        verifylist = [
            ('configuration', 'cfg'),
            ('instances', ['i1', 'i2']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_configuration.return_value = self._data
        self.client.apply_configuration.return_value = self._data

        class Inst(object):
            def __init__(self, id):
                self.id = id

        self.client.get_instance.side_effect = lambda s: Inst(s)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        get_calls = [
            mock.call('i1'),
            mock.call('i2')
        ]

        self.client.get_instance.assert_has_calls(get_calls)
        self.client.find_configuration.assert_called_with('cfg',
                                                          ignore_missing=False)

        self.client.apply_configuration.assert_called_once_with(
            self._data.id,
            instances=['i1', 'i2']
        )

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(self.data, list(data))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.dcs.v1 import restore_record
from otcextensions.tests.unit.osclient.dcs.v1 import fakes


class TestListRestoreRecords(fakes.TestDCS):

    objects = fakes.FakeRestoreRecord.create_multiple(3)
    inst = fakes.FakeInstance.create_one()

    columns = ('id', 'name', 'backup_id', 'progress', 'status', 'error_code')

    data = []

    for s in objects:
        data.append((
            s.id,
            s.name,
            s.backup_id,
            s.progress,
            s.status,
            s.error_code,
        ))

    def setUp(self):
        super(TestListRestoreRecords, self).setUp()

        self.cmd = restore_record.ListRestoreRecords(self.app, None)

        self.client.restore_records = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_list(self):
        arglist = [
            'inst'
        ]

        verifylist = [
            ('instance', 'inst')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.restore_records.side_effect = [
            self.objects
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.restore_records.assert_called_once_with(
            instance={'id': self.inst.id},
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_query(self):
        arglist = [
            'inst',
            '--limit', '1',
            '--start', '2',
            '--from_time', '3',
            '--to_time', '4',
        ]

        verifylist = [
            ('instance', 'inst'),
            ('limit', 1),
            ('start', 2),
            ('from_time', '3'),
            ('to_time', '4'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.restore_records.side_effect = [
            self.objects
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.restore_records.assert_called_once_with(
            instance={'id': self.inst.id},
            limit=1,
            start=2,
            begin_time='3',
            end_time='4'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestRecoverBackup(fakes.TestDCS):

    _data = fakes.FakeRestoreRecord.create_one()
    inst = fakes.FakeInstance.create_one()

    columns = ()

    data = (

    )

    def setUp(self):
        super(TestRecoverBackup, self).setUp()

        self.cmd = restore_record.RestoreBackup(self.app, None)

        self.client.restore_instance = mock.Mock()
        self.client.find_instance = mock.Mock()

    def test_backup_create(self):
        arglist = [
            'inst',
            '--backup', '2',
        ]
        verifylist = [
            ('instance', 'inst'),
            ('backup', '2'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.restore_instance.side_effect = [
            self._data
        ]
        self.client.find_instance.side_effect = [
            self.inst
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.restore_instance.assert_called_with(
            instance={'id': self.inst.id},
            backup_id='2',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from unittest.mock import call

import mock
from osc_lib import exceptions

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import visualization_job
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes

_COLUMNS = (
    "created_at",
    "duration",
    "id",
    "is_success",
    "job_desc",
    "job_id",
    "job_name",
    "name",
    "resource_id",
    "service_url",
    "status",
    "train_url",
)


class TestListVisualizationJobs(fakes.TestModelartsv1):
    objects = fakes.FakeVisualizationJob.create_multiple(3)

    column_list_headers = (
        "Job Id",
        "Job Name",
        "Created At",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.job_id,
                s.job_name,
                cli_utils.UnixTimestampFormatter(s.created_at),
            )
        )

    def setUp(self):
        super(TestListVisualizationJobs, self).setUp()

        self.cmd = visualization_job.ListVisualizationJobs(self.app, None)

        self.client.visualization_jobs = mock.Mock()
        self.client.api_mock = self.client.visualization_jobs

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--limit",
            "1",
            "--offset",
            "2",
            "--status",
            "10",
            "--sort-by",
            "job_name",
            "--order",
            "asc",
            "--search-content",
            "test-job-name",
            "--workspace-id",
            "0",
        ]

        verifylist = [
            ("limit", 1),
            ("offset", 2),
            ("status", 10),
            ("sort_by", "job_name"),
            ("order", "asc"),
            ("search_content", "test-job-name"),
            ("workspace_id", "0"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            limit=1,
            offset=2,
            status=10,
            sort_by="job_name",
            order="asc",
            search_content="test-job-name",
            workspace_id="0",
        )


class TestCreateVisualizationJob(fakes.TestModelartsv1):
    _data = fakes.FakeVisualizationJob.create_one()
    columns = _COLUMNS

    data = fakes.gen_data(_data, columns, visualization_job._formatters)

    def setUp(self):
        super(TestCreateVisualizationJob, self).setUp()

        self.cmd = visualization_job.CreateVisualizationJob(self.app, None)

        self.client.create_visualization_job = mock.Mock(
            return_value=self._data
        )

    def test_create(self):
        arglist = [
            "test-visualizationjob",
            "--description",
            "1",
            "--train-url",
            "2",
            "--job-type",
            "3",
            "--flavor",
            "4",
            "--auto-stop-duration",
            "5",
        ]
        verifylist = [
            ("name", "test-visualizationjob"),
            ("job_desc", "1"),
            ("train_url", "2"),
            ("job_type", "3"),
            ("flavor", "4"),
            ("auto_stop_duration", 5),
        ]
        # Verify cm is triggereg with default parameters
        self.check_parser(self.cmd, arglist, verifylist)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            "job_name": "test-visualizationjob",
            "train_url": "2",
            "job_type": "3",
            "flavor": {"code": "4"},
            "job_desc": "1",
            "schedule": {
                "type": "stop",
                "time_unit": "HOURS",
                "duration": 5,
            },
        }
        self.client.create_visualization_job.assert_called_with(**attrs)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateVisualizationJob(fakes.TestModelartsv1):
    _data = fakes.FakeVisualizationJob.create_one()

    columns = _COLUMNS

    data = fakes.gen_data(_data, columns)

    def setUp(self):
        super(TestUpdateVisualizationJob, self).setUp()

        self.cmd = visualization_job.UpdateVisualizationJob(self.app, None)

        self.client.update_visualization_job = mock.Mock(
            return_value=self._data
        )

    def test_update(self):
        arglist = [
            "1234",
            "--description",
            "New Description",
        ]
        verifylist = [
            ("jobId", "1234"),
            ("description", "New Description"),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.update_visualization_job.assert_called_with(
            "1234", "New Description"
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteVisualizationJob(fakes.TestModelartsv1):
    def setUp(self):
        super(TestDeleteVisualizationJob, self).setUp()

        self.client.delete_visualization_job = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = visualization_job.DeleteVisualizationJob(self.app, None)

    def test_delete(self):
        arglist = ["123"]

        verifylist = [
            ("jobId", [123]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)
        self.client.delete_visualization_job.assert_called_with(123)
        self.assertIsNone(result)

    def test_multiple_delete(self):
        arglist = [
            "1234",
            "9876",
        ]

        verifylist = [
            ("jobId", [1234, 9876]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        delete_calls = [call(1234), call(9876)]
        self.client.delete_visualization_job.assert_has_calls(delete_calls)
        self.assertIsNone(result)

    def test_multiple_delete_with_exception(self):
        arglist = [
            "1234",
            "5544",  # non-existing-job-id
        ]

        verifylist = [
            ("jobId", [1234, 5544]),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        delete_mock_results = [None, exceptions.CommandError]
        self.client.delete_visualization_job = mock.Mock(
            side_effect=delete_mock_results
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual(
                "1 of 2 visualization job(s) failed to delete.",
                str(e),
            )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create Image
"""

import openstack
from otcextensions import sdk

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)
attrs = {
    "name": "CentOS-7-x86_64-GenericCloud.qcow2",
    "description": "Create an image from a file in an OBS bucket",
    "image_url": "imsv2-extensions-test:CentOS-7-x86_64.qcow2",
    "os_version": "CentOS 7.0 64bit",
    "min_disk": 40,
    "image_tags": [{"key": "key2", "value": "value2"},
                   {"key": "key1", "value": "value1"}]
}
result = conn.imsv2.create_image(**attrs)
print(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DcsBool(object):
    val = False

    def __init__(self, value):
        if value.lower() == 'true':
            self.val = True
        else:
            self.val = False

    def __get__(self):
        return self.val

    def __set__(self, value):
        if value.lower() == 'true':
            self.val = True
        else:
            self.val = False

    def __eq__(self, other):
        return self.val == other


class Backup(resource.Resource):

    resources_key = 'backup_record_response'

    base_path = '/instances/%(instance_id)s/backups'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'limit', 'start', 'begin_time', 'end_time',
        begin_time='beginTime',
        end_time='endTime')

    # Properties
    instance_id = resource.URI('instance_id')
    #: Time at which the backup task is created.
    created_at = resource.Body('created_at')
    #:  Description of the DCS instance backup.
    description = resource.Body('remark')
    #: Error code returned if DCS instance backup fails.
    error_code = resource.Body('error_code')
    #: Backup ID
    id = resource.Body('backup_id', alternate_id=True)
    #: An indicator of whether restoration is supported. The
    #: value can be TRUE or FALSE.
    is_restorable = resource.Body('is_support_restore', type=DcsBool)
    #: Backup name
    name = resource.Body('backup_name')
    #: Time segment in which DCS instance backup was performed.
    period = resource.Body('period')
    #: Backup progress.
    progress = resource.Body('progress')
    #: Size of the backup file.
    #: Unit: byte.
    size = resource.Body('size', type=int)
    #: Backup status.
    #: * waiting: DCS instance backup is waiting to begin.
    #: * backuping: DCS instance backup is in progress.
    #: * succeed: DCS instance backup succeeded.
    #: * failed: DCS instance backup failed.
    #: * expired: The backup file has expired.
    #: * deleted: The backup file has been deleted manually.
    status = resource.Body('status')
    #: Backup type.
    #: * manual: manual backup.
    #: * auto: automatic backup.
    type = resource.Body('backup_type')
    #: Time at which DCS instance backup is completed.
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.dms.v1 import _base


class Queue(_base.Resource):

    resources_key = 'queues'

    base_path = '/queues'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'include_deadletter',
        'include_messages_num'
    )

    # Properties
    #: Created time
    #: *Type: int*
    created = resource.Body('created', type=int)
    #: Description for the queue.
    #: The value is a string of a maximum of 160 characters and cannot
    #: contain the angle brackets (<>).
    description = resource.Body('description')
    #: Queue Id
    id = resource.Body('id')
    #: Max consume count number
    #: *Type: int*
    #: Value range: 1–100.
    max_consume_count = resource.Body('max_consume_count', type=int)
    #: Queue name
    name = resource.Body('name')
    #: Queue mode:
    #: `NORMAL`: Standard queue, which supports high concurrency performance
    #: but cannot guarantee that messages are retrieved in the exact
    #: sequence as how they are received.
    #: `FIFO`: First-in-first-out (FIFO) queue, which guarantees that messages
    #: are retrieved in the exact sequence as how they are received.
    #: `KAFKA_HA`: High-reliability Kafka queue. All message replicas are
    #: flushed to a disk synchronously, ensuring message reliability.
    #: `KAFKA_HT`: High-throughput Kafka queue. All message replicas are
    #: flushed to a disk asynchronously, ensuring high performance.
    queue_mode = resource.Body('queue_mode')
    #: Redrive policy.
    #: Supported values: `enable`, `disable`. Default: `disable`
    redrive_policy = resource.Body('redrive_policy')
    #: Indicates the hours of storing messages in the Kafka queue.
    #: This parameter is valid only when queue_mode is set to KAFKA_HA or
    #: KAFKA_HT. Value range: 1–72.
    #: *Type: int*
    retention_hours = resource.Body('retention_hours', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Direct Connection Endpoint Group v2 action implementation"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListEndpointGroups(command.Lister):
    _description = _("List of Direct Connect Endpoint Groups.")
    columns = (
        'id',
        'name',
        'description',
        'project id',
        'type',
        'endpoints'
    )

    def get_parser(self, prog_name):
        parser = super(ListEndpointGroups, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_("Specifies the ID of the Direct Connect Endpoint Group.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the name of the Direct Connect Endpoint Group.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the Direct "
                   "Connect Endpoint Group.")
        )
        parser.add_argument(
            '--project_id',
            metavar='<project_id>',
            help=_("Specifies the project ID.")
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_("Specifies the type of the Direct Connect endpoints."
                   "The value can only be cidr.")
        )
        parser.add_argument(
            '--endpoints',
            metavar='<endpoint>',
            nargs='+',
            default=[],
            help=_("Specifies the list of the endpoints in a Direct Connect "
                   "Endpoint Group.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'id',
            'name',
            'description',
            'project_id',
            'type'
        ]

        attrs = {}
        endpoints = []
        if parsed_args.endpoints:
            for endpoint in parsed_args.endpoints:
                endpoints.append(endpoint)
        if endpoints:
            attrs['endpoints'] = endpoints
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        data = client.endpoint_groups(**attrs)

        table = (self.columns, (utils.get_dict_properties(s, self.columns)
                                for s in data))
        return table


class ShowEndpointGroup(command.ShowOne):
    _description = _("Show Direct Connection Endpoint Group.")

    def get_parser(self, prog_name):
        parser = super(ShowEndpointGroup, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint_group',
            metavar='<endpoint_group>',
            help=_("Specifies the endpoint group ID or name.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas
        obj = client.find_endpoint_group(parsed_args.endpoint_group)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateEndpointGroup(command.ShowOne):
    _description = _("Create new Direct Connection Endpoint Group.")

    def get_parser(self, prog_name):
        parser = super(CreateEndpointGroup, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the name of the Direct Connect "
                   "Endpoint Group.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the "
                   "Direct Connect Endpoint Group.")
        )
        parser.add_argument(
            'project_id',
            metavar='<project_id>',
            help=_("Specifies the project ID.")
        )
        parser.add_argument(
            'type',
            metavar='<type>',
            help=_("Specifies the type of the Direct Connect endpoints."
                   "The value can only be cidr.")
        )
        parser.add_argument(
            'endpoints',
            metavar='<endpoint>',
            nargs='+',
            default=[],
            help=_("Specifies the list of the endpoints in a Direct Connect "
                   "Endpoint Group.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'name',
            'description',
            'project_id',
            'type'
        ]
        attrs = {}
        endpoints = []
        if parsed_args.endpoints:
            for endpoint in parsed_args.endpoints:
                endpoints.append(endpoint)
        if endpoints:
            attrs['endpoints'] = endpoints
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_endpoint_group(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class UpdateEndpointGroup(command.ShowOne):
    _description = _("Update a Direct Connect Endpoint Group.")

    def get_parser(self, prog_name):
        parser = super(UpdateEndpointGroup, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint_group',
            metavar='<endpoint_group>',
            help=_("Specifies the Endpoint Group ID or name.")
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_("Specifies the name of the Direct Connect "
                   "Endpoint Group.")
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_("Provides supplementary information about the Direct "
                   "Connect Endpoint Group.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas

        args_list = [
            'name',
            'description'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        if parsed_args.endpoint_group:
            endpoint_group = client.find_endpoint_group(
                parsed_args.endpoint_group
            )
            obj = client.update_endpoint_group(
                endpoint_group.id, **attrs
            )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteEndpointGroup(command.Command):
    _description = _("Delete the Direct Connect Endpoint Group.")

    def get_parser(self, prog_name):
        parser = super(DeleteEndpointGroup, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint_group',
            metavar='<endpoint_group>',
            help=_("Direct Connect Endpoint Group to delete.")
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcaas
        if parsed_args.endpoint_group:
            endpoint_group = client.find_endpoint_group(
                parsed_args.endpoint_group)
            client.delete_endpoint_group(endpoint_group.id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestSignature(TestApiG):
    environment = None

    def setUp(self):
        super(TestSignature, self).setUp()
        self.create_gateway()
        self.gateway_id = TestSignature.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"
        self.attrs = {
            "name": "otce_signature_1",
            "sign_type": "aes",
            "sign_algorithm": "aes-256-cfb",
        }
        self.sign = self.client.create_signature(
            gateway=self.gateway_id,
            **self.attrs
        )

        self.addCleanup(
            self.client.delete_signature,
            gateway=self.gateway_id,
            sign=self.sign
        )
        self.addCleanup(self.delete_gateway())

    def test_list_signatures(self):
        sign = list(self.client.signatures(
            gateway=self.gateway_id))
        self.assertEqual(len(sign), 1)

    def test_update_signature(self):
        attrs = {
            "name": "otce_signature_1",
            "sign_type": "aes",
            "sign_algorithm": "aes-128-cfb",
        }
        updated = self.client.update_signature(
            gateway=self.gateway_id,
            sign=self.sign.id,
            **attrs
        )
        self.assertEqual(updated.sign_algorithm, attrs["sign_algorithm"])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import throttling_excluded as tx

EXAMPLE = {
    "call_limits": 200,
    "app_name": "app_demo",
    "object_name": "app_demo",
    "object_id": "356de8eb7a8742168586e5daf5339965",
    "throttle_id": "3437448ad06f4e0c91a224183116e965",
    "apply_time": "2020-08-04T02:40:56Z",
    "id": "a3e9ff8db55544ed9db91d8b048770c0",
    "app_id": "356de8eb7a8742168586e5daf5339965",
    "object_type": "APP"
}


class TestThrottlingPolicy(base.TestCase):

    def test_basic(self):
        sot = tx.ThrottlingExcludedPolicy()
        self.assertEqual(
            f'/apigw/instances/%(gateway_id)s/throttles/'
            f'%(throttle_id)s/throttle-specials',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('throttle_specials', sot.resources_key)

    def test_make_it(self):
        sot = tx.ThrottlingExcludedPolicy(**EXAMPLE)
        self.assertEqual(EXAMPLE['call_limits'], sot.call_limits)
        self.assertEqual(EXAMPLE['app_name'], sot.app_name)
        self.assertEqual(EXAMPLE['object_name'], sot.object_name)
        self.assertEqual(EXAMPLE['object_id'], sot.object_id)
        self.assertEqual(EXAMPLE['throttle_id'], sot.throttle_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts service v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def set_attributes_for_print(clusters):
    for cluster in clusters:
        cluster.allocatable_resources = {
            "cpu_cores": cluster.allocatable_cpu_cores,
            "memory": cluster.allocatable_memory,
            "gpus": cluster.allocatable_gpus,
        }
        yield cluster


class ListServiceClusters(command.Lister):
    _description = _("List dedicated clusters for service deployment.")
    columns = (
        "Cluster Id",
        "Cluster Name",
        "Created At",
        "Status",
        "Allocatable Resources",
        "Charging Mode",
        "Max Node Count",
        "Nodes",
        "Services Count",
    )

    def get_parser(self, prog_name):
        parser = super(ListServiceClusters, self).get_parser(prog_name)
        parser.add_argument(
            "--cluster-name",
            metavar="<cluster_name>",
            help=_("Cluster name."),
        )
        parser.add_argument(
            "--status",
            metavar="<status>",
            help=_("Cluster Status."),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Start page of the paging list. Default value: 0"),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Maximum number of records returned on each page. "
                "Default value: 1000"
            ),
        )
        parser.add_argument(
            "--sort-by",
            metavar="<sort_by>",
            help=_(
                "Sorting field. The options are as follows:"
                "\ncreated_at: default value"
                "\ncluster_name"
            ),
        )
        parser.add_argument(
            "--order",
            metavar="<order>",
            help=_(
                "Sorting mode. The default value is desc. Options:"
                "\n`asc`: ascending order"
                "\n`desc`: descending order"
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        args_list = (
            "cluster_name",
            "limit",
            "offset",
            "order",
            "sort_by",
            "status",
        )
        query_params = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        data = client.service_clusters(**query_params)
        if data:
            data = set_attributes_for_print(data)

        formatters = {
            "Created At": cli_utils.UnixTimestampFormatter,
            "Nodes": cli_utils.YamlFormat,
            "Allocatable Resources": cli_utils.YamlFormat,
            "Services Count": cli_utils.YamlFormat,
        }

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Update configurations of a service."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")
attr = {
    "description": "aha",
    "status": "running",
    "config": [
        {
            "model_id": "model-id",
            "weight": "100",
            "specification": "modelarts.vm.high.p3",
            "instance_count": 1,
        }
    ],
}
service_id = "281fc022-f0dd-4adc-abbd-97273ca5f4fd"
response = conn.modelartsv1.update_service(service_id, **attr)
print(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class Quota(resource.Resource):
    base_path = '/elb/quotas'

    # capabilities
    allow_fetch = True

    # Properties
    #: Specifies the certificate quota.
    certificate = resource.Body('certificate', type=int)
    #: Specifies the health check quota.
    healthmonitor = resource.Body('healthmonitor', type=int)
    #: Specifies the IP address group quota.
    ipgroup = resource.Body('ipgroup', type=int)
    #: Specifies the forwarding policy quota.
    l7policy = resource.Body('l7policy', type=int)
    #: Specifies the listener quota.
    listener = resource.Body('listener', type=int)
    #: Specifies the loadbalancer quota.
    loadbalancer = resource.Body('loadbalancer', type=int)
    #: Specifies the backend server quota.
    member = resource.Body('member', type=int)
    #: Specifies the quota of backend servers in a backend server group.
    members_per_pool = resource.Body('members_per_pool', type=int)
    #: Specifies the backend server group quota.
    pool = resource.Body('pool', type=int)
    #: Specifies the IP address group quota.
    project_id = resource.Body('project_id', type=str)

    @classmethod
    def get(cls, session):
        session = cls._get_session(session)
        url = cls.base_path
        response = session.get(url)
        resp = response.json()
        if 'error' in resp:
            return
        value = cls.existing(
            connection=session._get_connection(),
            **resp['quota'])
        return value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestStream(base.BaseFunctionalTest):

    def setUp(self):
        super(TestStream, self).setUp()
        self.lts = self.conn.lts
        self.log_group_name = 'teststream' + uuid.uuid4().hex[:8]
        self.log_stream_name = 'teststream' + uuid.uuid4().hex[:8]
        self.log_group = self.lts.create_group(
            log_group_name=self.log_group_name,
            ttl_in_days=5
        )
        self.assertIsNotNone(self.log_group)
        self.assertIsNotNone(self.log_group.id)

        attrs = {
            'log_group': self.log_group.id,
            'log_stream_name': self.log_stream_name,
            'ttl_in_days': 5
        }
        self.log_stream = self.lts.create_stream(**attrs)
        self.assertIsNotNone(self.log_stream)
        self.assertIsNotNone(self.log_stream.id)

    def test_list(self):
        objects = list(self.lts.streams(log_group=self.log_group.id))
        self.assertGreaterEqual(len(objects), 0)

    def tearDown(self):
        super(TestStream, self).tearDown()
        log_stream = self.lts.delete_stream(log_stream=self.log_stream.id,
                                            log_group=self.log_group.id,
                                            ignore_missing=False)
        self.assertIsNone(log_stream)
        log_stream = self.lts.delete_stream(log_stream=self.log_stream.id,
                                            log_group=self.log_group.id,
                                            ignore_missing=True)
        self.assertIsNone(log_stream)
        log_group = self.lts.delete_group(group=self.log_group.id)
        self.assertIsNone(log_group)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import devenv
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.DEVENV

EXAMPLE_CREATE = {
    "name": "notebook-d115",
    "description": "",
    "profile_id": "Ascend-Power-Engine 1.0(python3)",
    "flavor": "modelarts.kat1.xlarge",
    "spec": {
        "storage": {
            "location": {
                "path": "/aaaaaaaaa/output/",
            },
            "type": "obs",
        },
        "auto_stop": {"enable": True, "duration": 3600},
    },
    "workspace": {"id": "0"},
}


class TestDevenv(base.TestCase):
    def setUp(self):
        super(TestDevenv, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = devenv.Devenv()

        self.assertEqual("/demanager/instances", sot.base_path)
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("instances", sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

        self.assertDictEqual(
            {
                "ai_project": "ai_project_id",
                "dev_env_type": "de_type",
                "pool_id": "pool_id",
                "provision_type": "provision_type",
                "limit": "limit",
                "marker": "marker",
                "offset": "offset",
                "order": "order",
                "show_self": "show_self",
                "sort_by": "sortby",
                "status": "status",
                "workspace_id": "workspace_id",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = devenv.Devenv(**EXAMPLE)
        updated_sot_attrs = {
            "creation_timestamp": "created_at",
            "latest_update_timestamp": "updated_at",
            "de_type": "dev_env_type",
        }
        self.assertEqual(
            sot.profile.dev_env_type, EXAMPLE["profile"]["de_type"]
        )
        del EXAMPLE["profile"]["de_type"]
        for key, value in EXAMPLE.items():
            if key in updated_sot_attrs.keys():
                self.assertEqual(
                    getattr(sot, updated_sot_attrs[key]), EXAMPLE[key]
                )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

    def test_create_sot(self):
        sot = devenv.Devenv(**EXAMPLE_CREATE)
        assert_attributes_equal(self, sot, EXAMPLE_CREATE)

    def test_action(self):
        data = {"id": "mock-id"}
        sot = devenv.Devenv(**data)
        action = "start"
        json_body = {"action": action}
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = data
        response.headers = {}
        self.sess.post.return_value = response
        rt = sot._action(self.sess, action)
        self.sess.post.assert_called_with(
            "demanager/instances/%s/action" % sot.id,
            json=json_body,
            headers={
                "Accept": "application/json",
                "Content-type": "application/json",
            },
        )
        self.assertEqual(rt, sot)

    def test_start(self):
        sot = devenv.Devenv.existing(id=EXAMPLE["id"])
        sot._action = mock.Mock()
        sot.start(self.sess)
        sot._action.assert_called_with(self.sess, "start")

    def test_stop(self):
        sot = devenv.Devenv.existing(id=EXAMPLE["id"])
        sot._action = mock.Mock()
        sot.stop(self.sess)
        sot._action.assert_called_with(self.sess, "stop")
